export interface TestMessage {
    role: string
    content: string
}

export class TestCompletion {
    token: string = "r:60abef69e7cd8181b146ceaba1fdbf02"
    messageList: any = []
    stream: boolean = true;
    constructor(messageList: any) {
        this.messageList = messageList || this.messageList
    }
    
    async sendMessage(messageList?: null | Array<TestMessage>, onMessage?: (content: string) => void): Promise<any> {
        this.messageList = messageList || this.messageList
        let body = {
            "messages": this.messageList,
            "stream": this.stream,
            "model": "fmode-4.5-128k",
            "temperature": 0.5,
            "presence_penalty": 0,
            "frequency_penalty": 0,
            "token": "Bearer " + this.token
        }

        let response = await fetch("https://server.fmode.cn/api/apig/aigc/gpt/v1/chat/completions", {
            "headers": {
            },
            "body": JSON.stringify(body),
            "method": "POST",
            "mode": "cors",
            "credentials": "omit"
        });

        // 单次响应处理
        if (this.stream == false) {
            let data = await response.json()
            //console.log(data)
            let lastContent = data?.choices?.[0]?.message?.content
            return lastContent
        }

        // 流式处理
        if (!response.body) {
            throw new Error("No response body in stream mode");
        }

        const reader = response.body.getReader();
        const decoder = new TextDecoder("utf-8");
        let buffer = '';
        let accumulatedContent = "";

        try {
            while (true) {
                const { done, value } = await reader.read();
                if (done) break;

                buffer += decoder.decode(value, { stream: true });
                
                // 处理缓冲区中的所有完整行
                let lastNewline = buffer.lastIndexOf('\n');
                if (lastNewline === -1) continue; // 等待更多数据

                const completeLines = buffer.substring(0, lastNewline + 1);
                buffer = buffer.substring(lastNewline + 1);

                const lines = completeLines.split('\n').filter(line => line.trim() !== '');

                for (const line of lines) {
                    if (!line.startsWith('data:')) continue;
                    if (line.includes('[DONE]')) continue;

                    try {
                        const jsonStr = line.substring(5).trim();
                        if (!jsonStr) continue;

                        // 尝试解析JSON
                        let data;
                        try {
                            data = JSON.parse(jsonStr);
                        } catch (e) {
                            // 如果解析失败，可能是数据不完整，暂时保留到缓冲区
                            buffer = line + '\n' + buffer;
                            continue;
                        }

                        const content = data?.choices?.[0]?.delta?.content || '';
                        if (content) {
                            accumulatedContent += content;
                            if (onMessage) {
                                onMessage(accumulatedContent);
                            }
                        }
                    } catch (e) {
                        console.error("Error processing stream data:", e);
                    }
                }
            }

            // 处理缓冲区中剩余的数据
            if (buffer.trim()) {
                try {
                    const line = buffer.trim();
                    if (line.startsWith('data:') && !line.includes('[DONE]')) {
                        const jsonStr = line.substring(5).trim();
                        if (jsonStr) {
                            const data = JSON.parse(jsonStr);
                            const content = data?.choices?.[0]?.delta?.content || '';
                            if (content) {
                                accumulatedContent += content;
                                if (onMessage) {
                                    onMessage(accumulatedContent);
                                }
                            }
                        }
                    }
                } catch (e) {
                    console.error("Error processing remaining buffer:", e);
                }
            }
        } finally {
            reader.releaseLock();
        }

        return accumulatedContent;
    }
}

/**
 * 使用AI生成符合指定结构的JSON数据
 * @param prompt 任务要求的整体提示词
 * @param jsonSchema 期望的JSON结构描述（用于提示词）
 * @param example 可选的JSON示例（帮助AI理解格式）
 * @param onMessage 实时生成内容回调
 * @returns 解析后的JSON对象
 */
export async function completionJSON(
    prompt: string,
    jsonSchema: string,
    example: object | null = null,
    onMessage?: (content: string) => void
): Promise<any> {
    // 1. 构建提示词
    const JsonResultParsePrompt = `请严格按照以下要求生成JSON数据：
- 数据结构要求：${jsonSchema}
- 只能返回一个完整的JSON对象 确保JSON格式正确
- 注意返回的JSON格式每个KEY都有""包裹和Value之间都有:`;

    // 2. 初始化消息列表
    const messages: TestMessage[] = [
        {
            role: "user",
            content: prompt + JsonResultParsePrompt
        }
    ];

    // 3. 创建TestCompletion实例
    const completion = new TestCompletion(messages);
    
    // 4. 存储累积内容和JSON对象
    let fullContent = "";
    let jsonObject: any = null;
    
    // 5. 发送请求并处理流式响应
    const result = await completion.sendMessage(null, (content) => {
        fullContent = content;
        if (onMessage) {
            onMessage(content);
        }
    });
    
    // 6. 最终JSON提取
    try {
        jsonObject = extractJSON(fullContent);
        if (!jsonObject) {
            throw new Error("无法从响应中提取有效的JSON");
        }
    } catch (e) {
        console.error("JSON解析失败:", e);
        console.log("原始内容:", fullContent);
        throw new Error("生成的响应不符合JSON格式");
    }
    
    return jsonObject;
}

export function extractJSON(str: string) {
    // 首先尝试直接解析整个字符串
    try {
        const obj = JSON.parse(str);
        return obj;
    } catch (e) {
        // 如果直接解析失败，尝试提取JSON部分
    }

    // 提取JSON的备用方法
    let firstBrace = str.indexOf('{');
    let lastBrace = str.lastIndexOf('}');
    
    if (firstBrace === -1 || lastBrace === -1 || lastBrace <= firstBrace) {
        return null;
    }
    
    try {
        const jsonStr = str.substring(firstBrace, lastBrace + 1);
        return JSON.parse(jsonStr);
    } catch (e) {
        return null;
    }
}